@@ -289,7 +289,7 @@ class PatternReference(dict):
                 return False
 
     def is_violated(self, dict_obj):
-        """Check if new pattern reference doesnt violate with system reference
+        """Check if new pattern reference doesn't violate with system reference
 
         Parameters
         ----------
@@ -374,7 +374,7 @@ SYMBOL = SymbolCls()
 class TextPattern(str):
     """Use to convert text data to regex pattern
 
-    Parameters
+    Attributes
     ----------
     text (str): a text.
     as_is (bool): keeping text an AS-IS pattern.
@@ -431,18 +431,18 @@ class TextPattern(str):
         if self == '':
             return True
         else:
-            result = re.match(self, '')     # noqa
+            result = re.match(str(self), '')
             return bool(result)
 
     @property
     def is_empty_or_whitespace(self):
         is_empty = self.is_empty
-        is_ws = bool(re.match(self, ' '))   # noqa
+        is_ws = bool(re.match(str(self), ' '))
         return is_empty or is_ws
 
     @property
     def is_whitespace(self):
-        is_ws = bool(re.match(self, ' '))   # noqa
+        is_ws = bool(re.match(str(self), ' '))
         return is_ws
 
     @classmethod
@@ -628,10 +628,10 @@ class ElementPattern(str):
     """
     # patterns
     word_bound_pattern = r'word_bound(_left|_right|_raw)?$'
-    head_pattern = r'head(_raw|((_just)?_(whitespace|ws|space)(_plus)?))?$'
-    tail_pattern = r'tail(_raw|((_just)?_(whitespace|ws|space)(_plus)?))?$'
+    head_pattern = r'head(_raw|((_just)?_(whitespaces?|ws|spaces?)(_plus)?))?$'
+    tail_pattern = r'tail(_raw|((_just)?_(whitespaces?|ws|spaces?)(_plus)?))?$'
     repetition_pattern = r'repetition_\d*(_\d*)?$'
-    occurrence_pattern = r'({})(?P<is_phrase>_phrase)?_occurrences?$'.format(
+    occurrence_pattern = r'({})(?P<is_phrase>_(group|phrase))?_occurrences?$'.format(
         '|'.join([
             r'((?P<fda>\d+)_or_(?P<lda>\d+))',
             r'((?P<fdb>\d+)_or_(?P<ldb>more))',
@@ -640,6 +640,7 @@ class ElementPattern(str):
         ])
     )
     meta_data_pattern = r'^meta_data_\w+'
+    _variable = None
 
     def __new__(cls, text, as_is=False):
         cls._variable = VarCls()
@@ -807,7 +808,7 @@ class ElementPattern(str):
                 if arg == 'meta_data_raw':
                     'meta_data' not in lst and lst.append('meta_data')
                 else:
-                    cls._variable.option = arg.lstrip('meta_data_')     # noqa
+                    cls._variable.option = arg.lstrip('meta_data_')
             else:
                 match = re.match(or_pat, arg, flags=re.I)
                 if match:
@@ -910,7 +911,7 @@ class ElementPattern(str):
                 if arg == 'meta_data_raw':
                     'meta_data' not in lst and lst.append('meta_data')
                 else:
-                    cls._variable.option = arg.lstrip('meta_data_')  # noqa
+                    cls._variable.option = arg.lstrip('meta_data_')
             else:
                 match = re.match(or_pat, arg, flags=re.I)
                 if match:
@@ -1007,7 +1008,7 @@ class ElementPattern(str):
                 if arg == 'meta_data_raw':
                     'meta_data' not in lst and lst.append('meta_data')
                 else:
-                    cls._variable.option = arg.lstrip('meta_data_')     # noqa
+                    cls._variable.option = arg.lstrip('meta_data_')
             else:
                 match = re.match(or_pat, arg, flags=re.I)
                 if match:
@@ -1084,7 +1085,7 @@ class ElementPattern(str):
                 if arg == 'meta_data_raw':
                     'meta_data' not in lst and lst.append('meta_data')
                 else:
-                    cls._variable.option = arg.lstrip('meta_data_')     # noqa
+                    cls._variable.option = arg.lstrip('meta_data_')
             else:
                 match = re.match(or_pat, arg, flags=re.I)
                 if match:
@@ -1161,7 +1162,7 @@ class ElementPattern(str):
                 if arg == 'meta_data_raw':
                     'meta_data' not in lst and lst.append('meta_data')
                 else:
-                    cls._variable.option = arg.lstrip('meta_data_')     # noqa
+                    cls._variable.option = arg.lstrip('meta_data_')
             else:
                 match = re.match(or_pat, arg, flags=re.I)
                 if match:
@@ -1205,8 +1206,10 @@ class ElementPattern(str):
         if keyword != 'start':
             return False, ''
 
-        table = dict(space=r'^ *', space_plus=r'^ +',
-                     ws=r'^\s*', ws_plus=r'^\s+')
+        table = dict(space=r'^ *', spaces=r'^ +', space_plus=r'^ +',
+                     ws=r'^\s*', ws_plus=r'^\s+',
+                     whitespace=r'^\s*', whitespaces=r'^\s+',
+                     whitespace_plus=r'^\s+')
         pat = table.get(params, r'^')
         return True, pat
 
@@ -1226,8 +1229,10 @@ class ElementPattern(str):
         if keyword != 'end':
             return False, ''
 
-        table = dict(space=r' *$', space_plus=r' +$',
-                     ws=r'\s*$', ws_plus=r'\s+$')
+        table = dict(space=r' *$', spaces=r' +$', space_plus=r' +$',
+                     ws=r'\s*$', ws_plus=r'\s+$',
+                     whitespace=r'\s*$', whitespaces=r'\s+$',
+                     whitespace_plus=r'\s+$')
         pat = table.get(params, r'$')
         return True, pat
 
@@ -1320,8 +1325,8 @@ class ElementPattern(str):
         str: new pattern with variable name.
         """
         if name:
-            cls._variable.name = name       # noqa
-            cls._variable.pattern = pattern     # noqa
+            cls._variable.name = name
+            cls._variable.pattern = pattern
             if pattern.startswith('(') and pattern.endswith(')'):
                 sub_pat = pattern[1:-1]
                 if pattern.endswith('|)'):
@@ -1329,7 +1334,7 @@ class ElementPattern(str):
                 else:
                     try:
                         re.compile(sub_pat)
-                        cls._variable.pattern = sub_pat     # noqa
+                        cls._variable.pattern = sub_pat
                         new_pattern = '(?P<{}>{})'.format(name, sub_pat)
                     except Exception as ex:     # noqa
                         new_pattern = '(?P<{}>{})'.format(name, pattern)
@@ -1407,6 +1412,9 @@ class ElementPattern(str):
             elif head == 'head_space_plus' and not pattern.startswith(case4):
                 new_pattern = '{}{}'.format(case4, pattern)
                 cls._prepended_pattern = case4
+            elif head == 'head_spaces' and not pattern.startswith(case4):
+                new_pattern = '{}{}'.format(case4, pattern)
+                cls._prepended_pattern = case4
             elif head == 'head' and not pattern.startswith(case5):
                 new_pattern = '{}{}'.format(case5, pattern)
                 cls._prepended_pattern = case5
@@ -1422,18 +1430,27 @@ class ElementPattern(str):
             elif head == 'head_just_space_plus' and not pattern.startswith(case9):
                 new_pattern = '{}{}'.format(case9, pattern)
                 cls._prepended_pattern = case9
+            elif head == 'head_just_spaces' and not pattern.startswith(case9):
+                new_pattern = '{}{}'.format(case9, pattern)
+                cls._prepended_pattern = case9
             elif head == 'head_whitespace' and not pattern.startswith(case10):
                 new_pattern = '{}{}'.format(case10, pattern)
                 cls._prepended_pattern = case10
             elif head == 'head_whitespace_plus' and not pattern.startswith(case11):
                 new_pattern = '{}{}'.format(case11, pattern)
                 cls._prepended_pattern = case11
+            elif head == 'head_whitespaces' and not pattern.startswith(case11):
+                new_pattern = '{}{}'.format(case11, pattern)
+                cls._prepended_pattern = case11
             elif head == 'head_just_whitespace' and not pattern.startswith(case12):
                 new_pattern = '{}{}'.format(case12, pattern)
                 cls._prepended_pattern = case12
             elif head == 'head_just_whitespace_plus' and not pattern.startswith(case13):
                 new_pattern = '{}{}'.format(case13, pattern)
                 cls._prepended_pattern = case13
+            elif head == 'head_just_whitespaces' and not pattern.startswith(case13):
+                new_pattern = '{}{}'.format(case13, pattern)
+                cls._prepended_pattern = case13
             else:
                 new_pattern = pattern
             return new_pattern
@@ -1475,6 +1492,9 @@ class ElementPattern(str):
             elif tail == 'tail_space_plus' and not pattern.endswith(case4):
                 new_pattern = '{}{}'.format(pattern, case4)
                 cls._appended_pattern = case4
+            elif tail == 'tail_spaces' and not pattern.endswith(case4):
+                new_pattern = '{}{}'.format(pattern, case4)
+                cls._appended_pattern = case4
             elif tail == 'tail' and not pattern.endswith(case5):
                 new_pattern = '{}{}'.format(pattern, case5)
                 cls._appended_pattern = case5
@@ -1490,18 +1510,27 @@ class ElementPattern(str):
             elif tail == 'tail_just_space_plus' and not pattern.startswith(case9):
                 new_pattern = '{}{}'.format(pattern, case9)
                 cls._appended_pattern = case9
+            elif tail == 'tail_just_spaces' and not pattern.startswith(case9):
+                new_pattern = '{}{}'.format(pattern, case9)
+                cls._appended_pattern = case9
             elif tail == 'tail_whitespace' and not pattern.startswith(case10):
                 new_pattern = '{}{}'.format(pattern, case10)
                 cls._appended_pattern = case10
             elif tail == 'tail_whitespace_plus' and not pattern.startswith(case11):
                 new_pattern = '{}{}'.format(pattern, case11)
                 cls._appended_pattern = case11
+            elif tail == 'tail_whitespaces' and not pattern.startswith(case11):
+                new_pattern = '{}{}'.format(pattern, case11)
+                cls._appended_pattern = case11
             elif tail == 'tail_just_whitespace' and not pattern.startswith(case12):
                 new_pattern = '{}{}'.format(pattern, case12)
                 cls._appended_pattern = case12
             elif tail == 'tail_just_whitespace_plus' and not pattern.startswith(case13):
                 new_pattern = '{}{}'.format(pattern, case13)
                 cls._appended_pattern = case13
+            elif tail == 'tail_just_whitespaces' and not pattern.startswith(case13):
+                new_pattern = '{}{}'.format(pattern, case13)
+                cls._appended_pattern = case13
             else:
                 new_pattern = pattern
             return new_pattern
@@ -1544,7 +1573,7 @@ class ElementPattern(str):
         Parameters
         ----------
         lst (lst): a list of sub pattens
-        occurrence (str): a occurrence expression.  Default is empty.
+        occurrence (str): an occurrence expression.  Default is empty.
 
         Returns
         -------
@@ -1718,6 +1747,9 @@ class LinePattern(str):
     LinePatternError: raise an exception if pattern is invalid.
 
     """
+
+    _variables = None
+
     def __new__(cls, text, prepended_ws=False, appended_ws=False,
                 ignore_case=False):
         cls._variables = list()
@@ -1788,18 +1820,19 @@ class LinePattern(str):
 
         lst = []
         start = 0
+        m = None
         for m in re.finditer(r'\w+[(][^)]*[)]', line):
             pre_match = m.string[start:m.start()]
             if pre_match:
                 lst.append(TextPattern(pre_match))
             elm_pat = ElementPattern(m.group())
             if not elm_pat.variable.is_empty:
-                cls._variables.append(elm_pat.variable)     # noqa
+                cls._variables.append(elm_pat.variable)
             lst.append(elm_pat)
             start = m.end()
         else:
-            if start:
-                after_match = m.string[start:]      # noqa
+            if m and start:
+                after_match = m.string[start:]
                 if after_match:
                     lst.append(TextPattern(after_match))
 
